/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   This file builds a nested model out of two copies of Model_1.

   The scene tree is basically this.

                Model_2
                 /  \
               /      \
             /          \
        Model_1        Model_1

   With more detail, the scene tree looks like this. Remember that
   every node in the tree contains a matrix, some geometry (the
   list of line segments) and a list of sub-models, but the list
   of line segments may be empty, and the list of sub-models may
   be empty (the matrix cannot be "empty"; if you don't give it a
   value, it is the identity matrix, I).

               Model_2
              /       \
            /           \
       matrix          sub-models
                        /        \
                      /            \
                Model_1            Model_1
                /   \               /    \
              /       \           /        \
         matrix    geometry    matrix    geometry
           I                    ITSR
*/
public class Model_2 extends Model
{
   public Model_2(Color c1, Color c2)
   {
      super();

      Model m1 = new Model_1();
      Model m2 = new Model_1();
      m1.setColor(c1);
      m2.setColor(c2);

      m2.modelTranslate(1.0, -2.0-Math.sqrt(2.0), 0.0);
      m2.modelScale(0.5, 0.5, 1.0);
      m2.modelRotate(-45.0, 0.0, 0.0, 1.0);

      this.addSubModel(m1);
      this.addSubModel(m2);
   }
}
