/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
   Draw an interactive robot arm with shoulder, elbow, wrist, and finger joints.
*/
public class RobotArm2
{
   private static double shoulderRotation = 0.0;
   private static double    elbowRotation = 0.0;
   private static double    wristRotation = 0.0;
   private static double   fingerRotation = 0.0;

   private static double shoulderLength = 0.4;
   private static double    elbowLength = 0.3;
   private static double    wristLength = 0.2;
   private static double   fingerLength = 0.1;

   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the s/S keys to rotate the arm at the shoulder.");
      System.out.println("Use the e/E keys to rotate the arm at the elbow.");
      System.out.println("Use the w/W keys to rotate the arm at the wrist.");
      System.out.println("Use the f/F keys to rotate the arm at the finger.");

      /*
         Be sure to draw a picture of the (simple) tree that this code creates.
      */
      Model arm = new Model();
      arm.addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                         new Vertex(1, 0, 0)));
      arm.addSubModel( new Model() );
      arm.getSubModel(0).addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                                        new Vertex(1, 0, 0)));
      arm.getSubModel(0).addSubModel( new Model() );
      arm.getSubModel(0).getSubModel(0).addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                                                       new Vertex(1, 0, 0)));
      arm.getSubModel(0).getSubModel(0).addSubModel( new Model() );
      arm.getSubModel(0).getSubModel(0).getSubModel(0).addLineSegment(
              new LineSegment(new Vertex(0, 0, 0),
                              new Vertex(1, 0, 0)));

      arm.setColor( Color.blue );

      // Push the model away from where the camera is.
      arm.modelTranslate(0, 0, -1);

      // Initialize the model matrices for the sub models.
      arm.modelScale(shoulderLength, shoulderLength, 1);

      Model elbow = arm.getSubModel(0);
      elbow.modelTranslate(1, 0, 0);
      elbow.modelScale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1);

      Model wrist = elbow.getSubModel(0);
      wrist.modelTranslate(1, 0, 0);
      wrist.modelScale(wristLength/elbowLength, wristLength/elbowLength, 1);

      Model finger = wrist.getSubModel(0);
      finger.modelTranslate(1, 0, 0);
      finger.modelScale(elbowLength/fingerLength, elbowLength/fingerLength, 1);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the model to the Scene.
      scene.modelList.add(arm);


      // Create a Camera object.
      Camera camera = new Camera();

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 6", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
          //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
             //Pipeline.debug = ! Pipeline.debug;
               Pipeline.debug_visible = ! Pipeline.debug_visible;
             //Rasterize.debug = ! Rasterize.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            }
            else if ('s' == c)
            {
               shoulderRotation += 2.0;
            }
            else if ('S' == c)
            {
               shoulderRotation -= 2.0;
            }
            else if ('e' == c)
            {
               elbowRotation += 2.0;
            }
            else if ('E' == c)
            {
               elbowRotation -= 2.0;
            }
            else if ('w' == c)
            {
               wristRotation += 2.0;
            }
            else if ('W' == c)
            {
               wristRotation -= 2.0;
            }
            else if ('f' == c)
            {
               fingerRotation += 2.0;
            }
            else if ('F' == c)
            {
               fingerRotation -= 2.0;
            }

            // Set the model-to-view transformation matrix.
            arm.model2Identity();
            // Push the model away from where the camera is.
            arm.modelTranslate(0, 0, -1);

            // Set the model matrices for the sub models.
            arm.modelRotate(shoulderRotation, 0,0,1);
            arm.modelScale(shoulderLength, shoulderLength, 1);

            Model elbow = arm.getSubModel(0);
            elbow.model2Identity();
            elbow.modelTranslate(1, 0, 0);
            elbow.modelRotate(elbowRotation, 0,0,1);
            elbow.modelScale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1);

            Model wrist = elbow.getSubModel(0);
            wrist.model2Identity();
            wrist.modelTranslate(1, 0, 0);
            wrist.modelRotate(wristRotation, 0,0,1);
            wrist.modelScale(wristLength/elbowLength, wristLength/elbowLength, 1);

            Model finger = wrist.getSubModel(0);
            finger.model2Identity();
            finger.modelTranslate(1, 0, 0);
            finger.modelRotate(fingerRotation, 0,0,1);
            finger.modelScale(fingerLength/wristLength, fingerLength/wristLength, 1);

            // Render again.
            this.getFrameBuffer().clearFB(Color.green);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
          //System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.green);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
