/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
   Draw an interactive robot arm with a shoulder, two elbow, two wrist, and two finger joints.
*/
public class RobotArms1
{
   private static double shoulderRotation =  0.0;
   private static double   elbowRotation1 =  15.0;
   private static double   elbowRotation2 = -15.0;
   private static double   wristRotation1 =  0.0;
   private static double   wristRotation2 =  0.0;
   private static double  fingerRotation1 =  0.0;
   private static double  fingerRotation2 =  0.0;

   private static double shoulderLength = 0.4;
   private static double    elbowLength = 0.3;
   private static double    wristLength = 0.2;
   private static double   fingerLength = 0.1;

   public static void main(String[] args)
   {
      //System.out.println("Use the 'd' key to toggle debugging information on and off.");
      //System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the s/S keys to rotate the arm at the shoulder.");
      System.out.println("Use the e/E keys to rotate the arm at the elbow1.");
      System.out.println("Use the w/W keys to rotate the arm at the wrist1.");
      System.out.println("Use the f/F keys to rotate the arm at the finger1.");
      System.out.println("Use the d/D keys to rotate the arm at the elbow2.");
      System.out.println("Use the a/A keys to rotate the arm at the wrist2.");
      System.out.println("Use the c/C keys to rotate the arm at the finger2.");

      /*
         Be sure to draw a picture of the tree that this code creates.
      */
      Model arm = new Model();
      arm.addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                         new Vertex(1, 0, 0)));
      // two elbows
      arm.addSubModel( new Model() );
      arm.getSubModel(0).addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                                        new Vertex(1, 0, 0)));
      arm.addSubModel( new Model() );
      arm.getSubModel(1).addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                                        new Vertex(1, 0, 0)));
      // two wrists
      arm.getSubModel(0).addSubModel( new Model() );
      arm.getSubModel(0).getSubModel(0).addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                                                       new Vertex(1, 0, 0)));
      arm.getSubModel(1).addSubModel( new Model() );
      arm.getSubModel(1).getSubModel(0).addLineSegment(new LineSegment(new Vertex(0, 0, 0),
                                                                       new Vertex(1, 0, 0)));
      // two fingers
      arm.getSubModel(0).getSubModel(0).addSubModel( new Model() );
      arm.getSubModel(0).getSubModel(0).getSubModel(0).addLineSegment(
              new LineSegment(new Vertex(0, 0, 0),
                              new Vertex(1, 0, 0)));
      arm.getSubModel(1).getSubModel(0).addSubModel( new Model() );
      arm.getSubModel(1).getSubModel(0).getSubModel(0).addLineSegment(
              new LineSegment(new Vertex(0, 0, 0),
                              new Vertex(1, 0, 0)));

      arm.setColor( Color.blue );

      // Push the model away from where the camera is.
      arm.modelTranslate(0, 0, -1);

      // Initialize the model matrices for the sub models.
      arm.modelScale(shoulderLength, shoulderLength, 1);

      Model elbow1 = arm.getSubModel(0);
      elbow1.modelTranslate(1, 0, 0);
      elbow1.modelRotate(elbowRotation1, 0,0,1);
      elbow1.modelScale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1);

      Model elbow2 = arm.getSubModel(1);
      elbow2.modelTranslate(1, 0, 0);
      elbow2.modelRotate(elbowRotation2, 0,0,1);
      elbow2.modelScale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1);

      Model wrist1 = elbow1.getSubModel(0);
      wrist1.modelTranslate(1, 0, 0);
      wrist1.modelScale(wristLength/elbowLength, wristLength/elbowLength, 1);

      Model wrist2 = elbow2.getSubModel(0);
      wrist2.modelTranslate(1, 0, 0);
      wrist2.modelScale(wristLength/elbowLength, wristLength/elbowLength, 1);

      Model finger1 = wrist1.getSubModel(0);
      finger1.modelTranslate(1, 0, 0);
      finger1.modelScale(elbowLength/fingerLength, elbowLength/fingerLength, 1);

      Model finger2 = wrist2.getSubModel(0);
      finger2.modelTranslate(1, 0, 0);
      finger2.modelScale(elbowLength/fingerLength, elbowLength/fingerLength, 1);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the model to the Scene.
      scene.modelList.add(arm);


      // Create a Camera object.
      Camera camera = new Camera();

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 6", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
          //System.out.println( e );
            char c = e.getKeyChar();
            //if ('d' == c)
            //{
            // //Pipeline.debug = ! Pipeline.debug;
            //   Pipeline.debug_visible = ! Pipeline.debug_visible;
            // //Rasterize.debug = ! Rasterize.debug;
            // //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            //}
            //else if ('a' == c)
            //{
            //   Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            //}
            if ('s' == c)
            {
               shoulderRotation += 2.0;
            }
            else if ('S' == c)
            {
               shoulderRotation -= 2.0;
            }
            else if ('e' == c)
            {
               elbowRotation1 += 2.0;
            }
            else if ('E' == c)
            {
               elbowRotation1 -= 2.0;
            }
            else if ('w' == c)
            {
               wristRotation1 += 2.0;
            }
            else if ('W' == c)
            {
               wristRotation1 -= 2.0;
            }
            else if ('f' == c)
            {
               fingerRotation1 += 2.0;
            }
            else if ('F' == c)
            {
               fingerRotation1 -= 2.0;
            }
            else if ('d' == c)
            {
               elbowRotation2 += 2.0;
            }
            else if ('D' == c)
            {
               elbowRotation2 -= 2.0;
            }
            else if ('a' == c)
            {
               wristRotation2 += 2.0;
            }
            else if ('A' == c)
            {
               wristRotation2 -= 2.0;
            }
            else if ('c' == c)
            {
               fingerRotation2 += 2.0;
            }
            else if ('C' == c)
            {
               fingerRotation2 -= 2.0;
            }

            // Set the model-to-view transformation matrix.
            arm.model2Identity();
            // Push the model away from where the camera is.
            arm.modelTranslate(0, 0, -1);

            // Set the model matrices for the sub models.
            arm.modelRotate(shoulderRotation, 0,0,1);
            arm.modelScale(shoulderLength, shoulderLength, 1);

            Model elbow1 = arm.getSubModel(0);
            elbow1.model2Identity();
            elbow1.modelTranslate(1, 0, 0);
            elbow1.modelRotate(elbowRotation1, 0,0,1);
            elbow1.modelScale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1);

            Model elbow2 = arm.getSubModel(1);
            elbow2.model2Identity();
            elbow2.modelTranslate(1, 0, 0);
            elbow2.modelRotate(elbowRotation2, 0,0,1);
            elbow2.modelScale(elbowLength/shoulderLength, elbowLength/shoulderLength, 1);

            Model wrist1 = elbow1.getSubModel(0);
            wrist1.model2Identity();
            wrist1.modelTranslate(1, 0, 0);
            wrist1.modelRotate(wristRotation1, 0,0,1);
            wrist1.modelScale(wristLength/elbowLength, wristLength/elbowLength, 1);

            Model wrist2 = elbow2.getSubModel(0);
            wrist2.model2Identity();
            wrist2.modelTranslate(1, 0, 0);
            wrist2.modelRotate(wristRotation2, 0,0,1);
            wrist2.modelScale(wristLength/elbowLength, wristLength/elbowLength, 1);

            Model finger1 = wrist1.getSubModel(0);
            finger1.model2Identity();
            finger1.modelTranslate(1, 0, 0);
            finger1.modelRotate(fingerRotation1, 0,0,1);
            finger1.modelScale(fingerLength/wristLength, fingerLength/wristLength, 1);

            Model finger2 = wrist2.getSubModel(0);
            finger2.model2Identity();
            finger2.modelTranslate(1, 0, 0);
            finger2.modelRotate(fingerRotation2, 0,0,1);
            finger2.modelScale(fingerLength/wristLength, fingerLength/wristLength, 1);

            // Render again.
            this.getFrameBuffer().clearFB(Color.green);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
          //System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.green);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
